@@ -7,7 +7,7 @@
 
 int get_drive_width_windows(const char *s, int nchar)
 {
-    /* there are three types of absolute paths in windows
+    /* there are three types of absolute paths on windows
      *
      * there are those beginning with d:/ or some other letter
      * we call these drives
@@ -20,7 +20,7 @@ int get_drive_width_windows(const char *s, int nchar)
      * those filenames with R_ExpandFileName(), but for path.join() we don't
      * want to modify the inputs
      *
-     * unlike unix, a path beginning with / is NOT an absolute path.
+     * unlike unix-alikes, a path beginning with / is NOT an absolute path.
      * try this for yourself:
 
 setwd("C:/")
@@ -75,7 +75,7 @@ normalizePath("/path/to/file")
 
 
     /* s starts with d: or similar */
-    if (nchar >= 2 && *(s + 1) == ':') return 2;
+    if (nchar >= 2 && *s <= 0x7f && *(s + 1) == ':') return 2;
 
 
     if (*s == '~' &&             /* s starts with ~ */
@@ -169,9 +169,9 @@ normalizePath("/path/to/file")
 
 int get_drive_width_unix(const char *s, int nchar)
 {
-    /* similar to the above get_drive_width_windows() but specifically for unix,
-     * where a drivespec only really makes sense in terms of a network share
-     */
+    /* similar to the above get_drive_width_windows() but specifically for
+     * unix-alikes where a drivespec only really makes sense in terms of a
+     * network share */
 
 
     /* 5 characters is the minimum required for a network share
@@ -231,7 +231,7 @@ int is_abs_path_windows(const char *s)
 
 
     /* s starts with d:/ or similar */
-    if (nchar >= 3 && *(s + 1) == ':' &&
+    if (nchar >= 3 && *s <= 0x7f && *(s + 1) == ':' &&
         (*(s + 2) == '/' || *(s + 2) == '\\'))
     {
         return 1;
