@@ -1,19 +1,8 @@
+#include <R_ext/Rdynload.h>
 #include <R_ext/Visibility.h>
 #include "this.path.h"
 
 
-/*
-I used to have some C functions called using .Call(.),
-but I switched entirely to .External2(.) instead,
-so now this array is empty.
-
-I could remove this, but it's not causing any harm
- */
-static const R_CallMethodDef callRoutines[] = {
-    {NULL, NULL, 0}
-};
-
-
 static const R_ExternalMethodDef externalRoutines[] = {
 
 
@@ -29,6 +18,27 @@ static const R_ExternalMethodDef externalRoutines[] = {
     {"asargs", (DL_FUNC) &do_asargs, -1},
 
 
+    /* backports.c */
+
+
+#if R_version_less_than(3, 3, 0)
+    {"strrep"    , (DL_FUNC) &do_strrep    , 2},
+    {"startsWith", (DL_FUNC) &do_startsWith, 2},
+    {"endsWith"  , (DL_FUNC) &do_endsWith  , 2},
+#endif
+
+
+#if R_version_less_than(3, 2, 0)
+    {"direxists", (DL_FUNC) &do_direxists, 1},
+    {"lengths"  , (DL_FUNC) &do_lengths  , 2},
+#endif
+
+
+#if R_version_less_than(3, 1, 0)
+    {"anyNA", (DL_FUNC) &do_anyNA, -1},
+#endif
+
+
     /* basename2.c */
 
 
@@ -64,6 +74,11 @@ static const R_ExternalMethodDef externalRoutines[] = {
     /* hooks-for-namespace-events.c */
 
 
+    // {"utf8locale"  , (DL_FUNC) &do_utf8locale  , 0},
+    {"mbcslocale"  , (DL_FUNC) &do_mbcslocale  , 0},
+    // {"latin1locale", (DL_FUNC) &do_latin1locale, 0},
+    {"R_MB_CUR_MAX", (DL_FUNC) &do_R_MB_CUR_MAX, 0},
+
     {"onload"  , (DL_FUNC) &do_onload  , 2},
     {"onunload", (DL_FUNC) &do_onunload, 1},
 
@@ -84,6 +99,22 @@ static const R_ExternalMethodDef externalRoutines[] = {
     {"pathjoin"       , (DL_FUNC) &do_pathjoin       , 0},
 
 
+    /* pathsplit.c */
+
+
+    {"windowspathsplit", (DL_FUNC) &do_windowspathsplit, 1},
+    {"unixpathsplit"   , (DL_FUNC) &do_unixpathsplit   , 1},
+    {"pathsplit"       , (DL_FUNC) &do_pathsplit       , 1},
+
+    {"windowspathsplit1", (DL_FUNC) &do_windowspathsplit1, 1},
+    {"unixpathsplit1"   , (DL_FUNC) &do_unixpathsplit1   , 1},
+    {"pathsplit1"       , (DL_FUNC) &do_pathsplit1       , 1},
+
+    {"windowspathunsplit", (DL_FUNC) &do_windowspathunsplit, 0},
+    {"unixpathunsplit"   , (DL_FUNC) &do_unixpathunsplit   , 0},
+    {"pathunsplit"       , (DL_FUNC) &do_pathunsplit       , 0},
+
+
     /* promises.c */
 
 
@@ -91,6 +122,7 @@ static const R_ExternalMethodDef externalRoutines[] = {
     {"promiseisunevaluated"    , (DL_FUNC) &do_promiseisunevaluated    , -1},
     {"getpromisewithoutwarning", (DL_FUNC) &do_getpromisewithoutwarning, -1},
     {"prinfo"                  , (DL_FUNC) &do_prinfo                  , -1},
+    {"setthispathjupyter"      , (DL_FUNC) &do_setthispathjupyter      , -1},
 
 
     /* shfile.c */
@@ -112,17 +144,30 @@ static const R_ExternalMethodDef externalRoutines[] = {
 
     {"isclipboard"     , (DL_FUNC) &do_isclipboard     ,  1},
     {"thispath"        , (DL_FUNC) &do_thispath        ,  5},
+    {"localpath"       , (DL_FUNC) &do_localpath       ,  3},
     {"inittoolsrstudio", (DL_FUNC) &do_inittoolsrstudio, -1},
-    {"thispathrgui"    , (DL_FUNC) &do_thispathrgui    ,  5},
+    {"thispathrgui"    , (DL_FUNC) &do_thispathrgui    ,  6},
+
+
+    /* utils.c */
+
+
+#if R_version_less_than(3, 5, 0)
+    {"dotslength", (DL_FUNC) &do_dotslength, 0},
+#endif
+#if R_version_less_than(3, 2, 0)
+    {"isRegisteredNamespace", (DL_FUNC) &do_isRegisteredNamespace, 1},
+#endif
 
 
     /* wrapsource.c */
 
 
-    {"setprseen2"  , (DL_FUNC) &do_setprseen2  ,  1},
-    {"wrapsource"  , (DL_FUNC) &do_wrapsource  , 20},
-    {"insidesource", (DL_FUNC) &do_insidesource, 20},
-    {"setthispath" , (DL_FUNC) &do_setthispath , 20},
+    {"setprseen2"   , (DL_FUNC) &do_setprseen2   ,  1},
+    {"wrapsource"   , (DL_FUNC) &do_wrapsource   , 20},
+    {"insidesource" , (DL_FUNC) &do_insidesource , 21},
+    {"setthispath"  , (DL_FUNC) &do_setthispath  , 21},
+    {"unsetthispath", (DL_FUNC) &do_unsetthispath,  0},
 
 
     {NULL, NULL, 0}
@@ -131,7 +176,9 @@ static const R_ExternalMethodDef externalRoutines[] = {
 
 void attribute_visible R_init_this_path(DllInfo *dll)
 {
-    R_registerRoutines(dll, NULL, callRoutines, NULL, externalRoutines);
+    R_registerRoutines(dll, NULL, NULL, NULL, externalRoutines);
     R_useDynamicSymbols(dll, FALSE);
+#if R_version_at_least(3, 0, 0)
     R_forceSymbols(dll, TRUE);
+#endif
 }
