@@ -4,24 +4,65 @@
 
 #include <R.h>
 #include <Rinternals.h>
-#include <R_ext/Connections.h>
-#include <Rversion.h>
+#include "Rversiondefines.h"
+#include "translations.h"
+#include "defines.h"
+#include "thispathbackports.h"
 
 
-#if !defined(R_CONNECTIONS_VERSION)
-    #error why is R_CONNECTIONS_VERSION not defined????
-#elif R_CONNECTIONS_VERSION == 1
-    extern Rconnection R_GetConnection2(SEXP sConn);
-    extern Rconnection GetUnderlyingConnection(SEXP sConn);
-#else
-    #error this.path is only implemented for R_CONNECTIONS_VERSION 1
+#if R_version_at_least(3, 3, 0)
+    #if defined(R_THIS_PATH_DEFINES)
+        #include <R_ext/Connections.h>
+        #if !defined(R_CONNECTIONS_VERSION)
+        #elif R_CONNECTIONS_VERSION == 1
+            #define R_CONNECTIONS_VERSION_1
+            extern Rconnection R_GetConnection(SEXP sConn);
+        #endif
+    #endif
 #endif
 
 
-#include "translations.h"
+#include "symbols.h"
 
 
+#if R_version_less_than(3, 1, 0)
+#if R_version_less_than(3, 0, 0)
+#define NAMEDMAX 2
+#define NO_REFERENCES(x) (NAMED(x) == 0)
+#define MAYBE_REFERENCED(x) (! NO_REFERENCES(x))
+#define MARK_NOT_MUTABLE(x) SET_NAMED(x, NAMEDMAX)
+#endif
+#define INCREMENT_NAMED(x) do {                                \
+    SEXP _x_ = (x);                                            \
+    if (NAMED(_x_) != NAMEDMAX)                                \
+        SET_NAMED(_x_, NAMED(_x_) + 1);                        \
+} while (0)
+#endif
+
+
+#if R_version_less_than(3, 5, 0)
+#define ENSURE_NAMEDMAX(_x_) SET_NAMED((_x_), NAMEDMAX)
+#else
+extern void (ENSURE_NAMEDMAX)(SEXP x);
+#endif
+
+
+void INCREMENT_NAMED_defineVar(SEXP symbol, SEXP value, SEXP rho);
+void MARK_NOT_MUTABLE_defineVar(SEXP symbol, SEXP value, SEXP rho);
+
+
+#if defined(R_THIS_PATH_DEFINES) && R_version_at_least(3, 0, 0)
+extern Rboolean R_Visible;
+#define set_R_Visible(v) (R_Visible = ((v) ? TRUE : FALSE))
+#else
 #define set_R_Visible(v) (eval((v) ? R_NilValue : lang1(invisibleSymbol), R_BaseEnv))
+#endif
+
+
+extern SEXP R_shallow_duplicate_attr(SEXP x);
+extern SEXP installTrChar(SEXP x);
+
+
 #define streql(str1, str2) (strcmp((str1), (str2)) == 0)
 
 
@@ -41,7 +82,21 @@ extern int IS_LATIN1(SEXP x);
 extern int IS_ASCII(SEXP x);
 
 
-#if R_VERSION >= R_Version(4, 1, 0)
+extern void R_removeVarFromFrame(SEXP name, SEXP env);
+extern void removeFromFrame(SEXP *names, SEXP env);
+
+
+extern void UNIMPLEMENTED_TYPEt(const char *s, SEXPTYPE t);
+extern void UNIMPLEMENTED_TYPE(const char *s, SEXP x);
+
+
+#if R_version_less_than(3, 1, 0)
+extern SEXP lazy_duplicate(SEXP s);
+extern SEXP shallow_duplicate(SEXP s);
+#endif
+
+
+#if R_version_at_least(4, 1, 0)
 extern int IS_UTF8(SEXP x);
 #else
 #define IS_UTF8(x) (getCharCE((x)) == CE_UTF8)
@@ -53,9 +108,6 @@ extern int SET_CACHED(SEXP x);
 extern int IS_CACHED(SEXP x);
 
 
-extern void (ENSURE_NAMEDMAX)(SEXP x);
-
-
 extern SEXP R_getNSValue(SEXP call, SEXP ns, SEXP name, int exported);
 
 
@@ -65,18 +117,17 @@ extern const char *EncodeChar(SEXP x);
 extern void R_LockBinding(SEXP sym, SEXP env);
 
 
-#include "symbols.h"
-#include "requirethispathhelper.h"
-
-
 extern SEXP getInFrame(SEXP sym, SEXP env, int unbound_ok);
 
 
 extern SEXP as_environment_char(const char *what);
 
 
+#if defined(R_CONNECTIONS_VERSION_1)
 extern SEXP summaryconnection(Rconnection Rcon);
-extern SEXP summaryconnection2(SEXP sConn);
+#else
+extern SEXP summaryconnection(SEXP sConn);
+#endif
 
 
 extern SEXP errorCondition (const char *msg, SEXP call, const char **cls, int n, int nfields);
@@ -96,8 +147,11 @@ extern SEXP simpleError(const char *msg, SEXP call);
     "this.path::thisPathNotExistsError"
 
 
+#if defined(R_CONNECTIONS_VERSION_1)
 extern SEXP thisPathUnrecognizedConnectionClassError(SEXP call, Rconnection Rcon);
-extern SEXP thisPathUnrecognizedConnectionClassError2(SEXP call, SEXP summary);
+#else
+extern SEXP thisPathUnrecognizedConnectionClassError(SEXP call, SEXP summary);
+#endif
 extern SEXP thisPathUnrecognizedMannerError         (SEXP call);
 extern SEXP thisPathNotImplementedError             (const char *msg, SEXP call);
 extern SEXP thisPathNotExistsError                  (const char *msg, SEXP call);
@@ -125,19 +179,13 @@ extern void stop(SEXP cond);
 #endif
 
 
-extern SEXP _assign(SEXP file, SEXP frame);
-extern void assign_default(SEXP file, SEXP frame, SEXP rho);
+extern void assign_done(SEXP frame);
+extern void assign_default(SEXP file, SEXP frame, Rboolean check_not_directory);
 extern void assign_null(SEXP frame);
-extern void assign_chdir(SEXP file, SEXP owd, SEXP frame, SEXP rho);
-extern void assign_file_uri(SEXP ofile, SEXP file, SEXP frame, SEXP rho);
-extern void assign_file_uri2(SEXP description, SEXP frame, SEXP rho);
-extern void assign_url(SEXP ofile, SEXP file, SEXP frame, SEXP rho);
-
-
-#define assign_done(frame) do {                                \
-    defineVar(thispathdoneSymbol, R_NilValue, (frame));        \
-    R_LockBinding(thispathdoneSymbol, (frame));                \
-} while (0)
+extern void assign_chdir(SEXP file, SEXP owd, SEXP frame);
+extern void assign_file_uri(SEXP ofile, SEXP file, SEXP frame, Rboolean check_not_directory);
+extern void assign_file_uri2(SEXP description, SEXP frame, Rboolean check_not_directory);
+extern void assign_url(SEXP ofile, SEXP file, SEXP frame);
 
 
 #define isurl(url) (                                           \
@@ -148,24 +196,40 @@ extern void assign_url(SEXP ofile, SEXP file, SEXP frame, SEXP rho);
                    )
 
 
-#define get_description_from_Rconnection(Rcon)                 \
-    (((Rcon)->enc == CE_UTF8) ? mkCharCE((Rcon)->description, CE_UTF8) : mkChar((Rcon)->description))
-#define get_description_from_summary(summary)                  \
-    STRING_ELT(VECTOR_ELT((summary), 0), 0)
-#define get_class_from_Rconnection(Rcon) ((Rcon)->class)
-#define get_class_from_summary(summary)                        \
-    CHAR(STRING_ELT(VECTOR_ELT((summary), 1), 0))
-
-
-#if defined(R_VERSION) && R_VERSION >= R_Version(3, 3, 0)
-#define notimplementedforgzcon "this.path() not implemented for a gzcon()\n try installing 'this.path.helper':\n utils::install.packages(\"this.path.helper\", repos =\n     \"https://raw.githubusercontent.com/ArcadeAntics/PACKAGES\")"
+#if defined(R_CONNECTIONS_VERSION_1)
+#define get_connection_description(Rcon) mkCharCE((Rcon)->description, ((Rcon)->enc == CE_UTF8) ? CE_UTF8 : CE_NATIVE)
+#define get_connection_class(Rcon) ((Rcon)->class)
 #else
-#define notimplementedforgzcon "this.path() not implemented for a gzcon()"
+#define get_connection_description(summary) STRING_ELT(VECTOR_ELT((summary), 0), 0)
+#define get_connection_class(summary) CHAR(STRING_ELT(VECTOR_ELT((summary), 1), 0))
 #endif
 
 
-#define checkfile(call, rho, sym, ofile, frame, forcepromise,  \
-    assign_returnvalue,                                        \
+#if defined(R_CONNECTIONS_VERSION_1)
+typedef struct gzconn {
+    Rconnection con;
+} *Rgzconn;
+#define get_description_and_class                              \
+            Rconnection Rcon = R_GetConnection(ofile);         \
+            if (Rcon->isGzcon) {                               \
+                Rcon = (((Rgzconn)(Rcon->private))->con);      \
+                if (Rcon->isGzcon) error("%s; should never happen, please report!", _("invalid connection"));\
+            }                                                  \
+            SEXP description = get_connection_description(Rcon);\
+            const char *klass = get_connection_class(Rcon)
+#define Rcon_or_summary(Rcon, summary) Rcon
+#else
+#define get_description_and_class                              \
+            SEXP summary = summaryconnection(ofile);           \
+            SEXP description = get_connection_description(summary);\
+            const char *klass = get_connection_class(summary); \
+            if (streql(klass, "gzcon")) error("this.path() not implemented for a gzcon()")
+#define Rcon_or_summary(Rcon, summary) summary
+#endif
+
+
+#define checkfile(call, sym, ofile, frame, check_not_directory,\
+    forcepromise, assign_returnvalue,                          \
     maybe_chdir, getowd, hasowd,                               \
     character_only, conv2utf8, allow_blank_string,             \
     allow_clipboard, allow_stdin, allow_url, allow_file_uri,   \
@@ -177,7 +241,7 @@ extern void assign_url(SEXP ofile, SEXP file, SEXP frame, SEXP rho);
 {                                                              \
     if (TYPEOF(ofile) == STRSXP) {                             \
         if (LENGTH(ofile) != 1)                                \
-            errorcall(call, "invalid '%s', must be a character string", EncodeChar(PRINTNAME(sym)));\
+            errorcall(call, "'%s' must be a character string", EncodeChar(PRINTNAME(sym)));\
         SEXP file = STRING_ELT(ofile, 0);                      \
         if (file == NA_STRING)                                 \
             errorcall(call, "invalid '%s', must not be NA", EncodeChar(PRINTNAME(sym)));\
@@ -220,7 +284,7 @@ extern void assign_url(SEXP ofile, SEXP file, SEXP frame, SEXP rho);
         }                                                      \
         else if (!ignore_url && isurl(url)) {                  \
             if (allow_url) {                                   \
-                assign_url(ofile, file, frame, rho);           \
+                assign_url(ofile, file, frame);                \
                 if (assign_returnvalue)                        \
                     returnvalue = PROTECT(ofile);              \
             }                                                  \
@@ -229,7 +293,7 @@ extern void assign_url(SEXP ofile, SEXP file, SEXP frame, SEXP rho);
         }                                                      \
         else if (!ignore_file_uri && strncmp(url, "file://", 7) == 0) {\
             if (allow_file_uri) {                              \
-                assign_file_uri(ofile, file, frame, rho);      \
+                assign_file_uri(ofile, file, frame, check_not_directory);\
                 if (assign_returnvalue) {                      \
                     returnvalue = PROTECT(shallow_duplicate(ofile));\
                     SET_STRING_ELT(returnvalue, 0, STRING_ELT(getInFrame(thispathfileSymbol, frame, FALSE), 0));\
@@ -244,11 +308,11 @@ extern void assign_url(SEXP ofile, SEXP file, SEXP frame, SEXP rho);
             if (maybe_chdir) {                                 \
                 SEXP owd = getowd;                             \
                 if (hasowd)                                    \
-                    assign_chdir(ofile, owd, frame, rho);      \
+                    assign_chdir(ofile, owd, frame);           \
                 else                                           \
-                    assign_default(ofile, frame, rho);         \
+                    assign_default(ofile, frame, check_not_directory);\
             }                                                  \
-            else assign_default(ofile, frame, rho);            \
+            else assign_default(ofile, frame, check_not_directory);\
             if (assign_returnvalue) {                          \
                 returnvalue = PROTECT(shallow_duplicate(ofile));\
                 SET_STRING_ELT(returnvalue, 0, STRING_ELT(getInFrame(thispathfileSymbol, frame, FALSE), 0));\
@@ -259,45 +323,25 @@ extern void assign_url(SEXP ofile, SEXP file, SEXP frame, SEXP rho);
     }                                                          \
     else {                                                     \
         if (character_only)                                    \
-            errorcall(call, "invalid '%s', must be a character string", EncodeChar(PRINTNAME(sym)));\
+            errorcall(call, "'%s' must be a character string", EncodeChar(PRINTNAME(sym)));\
         else if (!inherits(ofile, "connection"))               \
             errorcall(call, "invalid '%s', must be a string or connection", EncodeChar(PRINTNAME(sym)));\
         else {                                                 \
-            SEXP summary = NULL;                               \
-            Rconnection Rcon = NULL;                           \
-            SEXP description = NULL;                           \
-            const char *klass = NULL;                          \
-            int thispathhelper_loaded = (findVarInFrame(R_NamespaceRegistry, thispathhelperSymbol) != R_UnboundValue);\
-            if (!thispathhelper_loaded) {                      \
-                SEXP summary = summaryconnection2(ofile);      \
-                description = get_description_from_summary(summary);\
-                klass = get_class_from_summary(summary);       \
-                if (streql(klass, "gzcon")) {                  \
-                    requirethispathhelper;                     \
-                    thispathhelper_loaded = (findVarInFrame(R_NamespaceRegistry, thispathhelperSymbol) != R_UnboundValue);\
-                    if (!thispathhelper_loaded) error(notimplementedforgzcon);\
-                }                                              \
-            }                                                  \
-            if (thispathhelper_loaded) {                       \
-                Rcon = GetUnderlyingConnection(ofile);         \
-                if (Rcon->isGzcon) error("invalid connection; should never happen, please report!");\
-                description = get_description_from_Rconnection(Rcon);\
-                klass = get_class_from_Rconnection(Rcon);      \
-            }                                                  \
+            get_description_and_class;                         \
             if (streql(klass, "file"  ) ||                     \
                 streql(klass, "gzfile") ||                     \
                 streql(klass, "bzfile") ||                     \
                 streql(klass, "xzfile") ||                     \
                 streql(klass, "fifo"  ))                       \
             {                                                  \
-                assign_file_uri2(description, frame, rho);     \
+                assign_file_uri2(description, frame, check_not_directory);\
                 if (forcepromise) getInFrame(thispathfileSymbol, frame, FALSE);\
             }                                                  \
             else if (streql(klass, "url-libcurl") ||           \
                      streql(klass, "url-wininet"))             \
             {                                                  \
                 if (allow_url) {                               \
-                    assign_url(ScalarString(description), description, frame, rho);\
+                    assign_url(ScalarString(description), description, frame);\
                     if (forcepromise) getInFrame(thispathfileSymbol, frame, FALSE);\
                 }                                              \
                 else                                           \
@@ -321,15 +365,14 @@ extern void assign_url(SEXP ofile, SEXP file, SEXP frame, SEXP rho);
                  */                                            \
                 if (allow_unz) {                               \
                     SEXP tmp = thisPathInZipFileError(R_NilValue, description);\
-                    INCREMENT_NAMED(tmp);                      \
-                    defineVar(thispatherrorSymbol, tmp, frame);\
+                    INCREMENT_NAMED_defineVar(thispatherrorSymbol, tmp, frame);\
                     R_LockBinding(thispatherrorSymbol, frame); \
                     tmp = ScalarString(description);           \
-                    INCREMENT_NAMED(tmp);                      \
-                    defineVar(thispathformsgSymbol, tmp, frame);\
+                    INCREMENT_NAMED_defineVar(thispathformsgSymbol, tmp, frame);\
                     R_LockBinding(thispathformsgSymbol, frame);\
                     defineVar(thispathassocwfileSymbol, R_NilValue, frame);\
                     R_LockBinding(thispathassocwfileSymbol, frame);\
+                    assign_done(frame);                        \
                 }                                              \
                 else                                           \
                     errorcall(call, "invalid '%s', cannot be a unz connection", EncodeChar(PRINTNAME(sym)));\
@@ -387,18 +430,13 @@ extern void assign_url(SEXP ofile, SEXP file, SEXP frame, SEXP rho);
                        know if this connection has an          \
                        associated file                         \
                      */                                        \
-                    SEXP tmp;                                  \
-                    if (thispathhelper_loaded)                 \
-                        tmp = thisPathUnrecognizedConnectionClassError(R_NilValue, Rcon);\
-                    else                                       \
-                        tmp = thisPathUnrecognizedConnectionClassError2(R_NilValue, summary);\
-                    INCREMENT_NAMED(tmp);                      \
-                    defineVar(thispatherrorSymbol, tmp, frame);\
+                    SEXP tmp = thisPathUnrecognizedConnectionClassError(R_NilValue, Rcon_or_summary(Rcon, summary));\
+                    INCREMENT_NAMED_defineVar(thispatherrorSymbol, tmp, frame);\
                     R_LockBinding(thispatherrorSymbol, frame); \
                     tmp = ScalarString(description);           \
-                    INCREMENT_NAMED(tmp);                      \
-                    defineVar(thispathformsgSymbol, tmp, frame);\
+                    INCREMENT_NAMED_defineVar(thispathformsgSymbol, tmp, frame);\
                     R_LockBinding(thispathformsgSymbol, frame);\
+                    assign_done(frame);                        \
                 }                                              \
                 else                                           \
                     errorcall(call, "invalid '%s', cannot be a connection of class '%s'",\
@@ -407,13 +445,12 @@ extern void assign_url(SEXP ofile, SEXP file, SEXP frame, SEXP rho);
         }                                                      \
         if (assign_returnvalue) returnvalue = PROTECT(ofile);  \
     }                                                          \
-    assign_done(frame);                                        \
     set_R_Visible(1);                                          \
 }
 
 
 #define sys_call(which, rho) eval(lang2(sys_callSymbol, (which)), (rho))
-#define getCurrentCall(rho) sys_call(lang1(sys_nframeSymbol), (rho))
+#define getCurrentCall(rho) eval(lang1(sys_callSymbol), (rho))
 
 
 /* doesn't work in general, for example sys.function() duplicates its return value */
@@ -452,30 +489,15 @@ extern Rboolean init_tools_rstudio(Rboolean skipCheck);
     ((gui_rstudio != -1) ? (gui_rstudio) : (gui_rstudio = asLogical(getInFrame(gui_rstudioSymbol, mynamespace, FALSE))))
 
 
-extern int maybe_in_shell;
+extern int maybe_unembedded_shell;
 
 
-#define is_maybe_in_shell                                      \
-    ((maybe_in_shell == -1) ? (maybe_in_shell = asLogical(getInFrame(maybe_in_shellSymbol, mynamespace, FALSE))) : (maybe_in_shell))
+#define is_maybe_unembedded_shell                              \
+    ((maybe_unembedded_shell != -1) ? (maybe_unembedded_shell) : (maybe_unembedded_shell = asLogical(getInFrame(maybe_unembedded_shellSymbol, mynamespace, FALSE))))
 
 
 #define get_debugSource                                        \
-    ((has_tools_rstudio) ? getInFrame(debugSourceSymbol, mynamespace, FALSE) : R_NilValue)
-
-
-#define get_source_file(name)                                  \
-    (((name) = (findVarInFrame(R_NamespaceRegistry, testthatSymbol) != R_UnboundValue)) ?\
-        (R_getNSValue(R_NilValue, testthatSymbol, source_fileSymbol, TRUE)) :\
-        (R_NilValue))
-
-
-#define get_knit(name)                                         \
-    (((name) = (findVarInFrame(R_NamespaceRegistry, knitrSymbol) != R_UnboundValue)) ?\
-        (R_getNSValue(R_NilValue, knitrSymbol, knitSymbol, TRUE)) :\
-        (R_NilValue))
-
-
-#define get_wrap_source (getInFrame(wrap_sourceSymbol, mynamespace, FALSE))
+    ((has_tools_rstudio) ? getInFrame(debugSourceSymbol, mynamespace, FALSE) : R_UnboundValue)
 
 
 extern SEXP mynamespace;
