@@ -3,9 +3,11 @@ main <- function ()
     FILE <- tempfile(fileext = ".R")
     on.exit(unlink(FILE), add = TRUE)
     this.path:::write.code(file = FILE, {
-        invisible(loadNamespace("microbenchmark"))
-        invisible(loadNamespace("this.path"))
-        print(this.path:::faster.subsequent.times.test())
+        if (requireNamespace("microbenchmark")) {
+            invisible(loadNamespace("microbenchmark"))
+            invisible(loadNamespace("this.path"))
+            print(this.path:::faster.subsequent.times.test())
+        } else cat("\npackage {microbenchmark} is not available :(\n")
     })
     cat("\n")
     this.path:::.Rscript(c("--default-packages=NULL", "--vanilla", FILE))
